@@ -106,7 +106,7 @@ send_response_in_thread_func (GTask        *task,
           g_autofree char *ruri = NULL;
           g_autoptr(GError) error = NULL;
 
-          ruri = register_document (uris[i], request->app_id, for_save, writable, &error);
+          ruri = register_document (uris[i], xdp_app_info_get_id (request->app_info), for_save, writable, &error);
           if (ruri == NULL)
             {
               g_warning ("Failed to register %s: %s", uris[i], error->message);
@@ -396,7 +396,7 @@ handle_open_file (XdpFileChooser *object,
                   GVariant *arg_options)
 {
   Request *request = request_from_invocation (invocation);
-  const char *app_id = request->app_id;
+  const char *app_id = xdp_app_info_get_id (request->app_info);
   g_autoptr(GError) error = NULL;
   g_autoptr(XdpImplRequest) impl_request = NULL;
   GVariantBuilder options;
@@ -509,7 +509,7 @@ handle_save_file (XdpFileChooser *object,
                   GVariant *arg_options)
 {
   Request *request = request_from_invocation (invocation);
-  const char *app_id = request->app_id;
+  const char *app_id = xdp_app_info_get_id (request->app_info);
   g_autoptr(GError) error = NULL;
   XdpImplRequest *impl_request;
   GVariantBuilder options;
