@@ -177,7 +177,7 @@ is_sandboxed (GDesktopAppInfo *info)
   return strstr (exec, "flatpak run ") != NULL;
 }
 
-static void
+static gboolean
 launch_application_with_uri (const char *choice_id,
                              const char *uri,
                              const char *parent_window,
@@ -191,8 +191,15 @@ launch_application_with_uri (const char *choice_id,
 
   if (is_sandboxed (info))
     {
+      g_autoptr(GError) error = NULL;
+
       g_debug ("registering %s for %s", uri, choice_id);
-      ruri = register_document (uri, choice_id, FALSE, writable, NULL);
+      ruri = register_document (uri, choice_id, FALSE, writable, &error);
+      if (ruri == NULL)
+        {
+          g_warning ("Error registering %s for %s: %s", uri, choice_id, error->message);
+          return FALSE;
+        }
     }
   else
     ruri = g_strdup (uri);
@@ -204,6 +211,8 @@ launch_application_with_uri (const char *choice_id,
 
   g_debug ("launching %s %s", choice_id, ruri);
   g_app_info_launch_uris (G_APP_INFO (info), &uris, context, NULL);
+
+  return TRUE;
 }
 
 static void
@@ -287,8 +296,8 @@ send_response_in_thread_func (GTask *task,
       writable = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (request), "writable"));
       content_type = (const char *)g_object_get_data (G_OBJECT (request), "content-type");
 
-      launch_application_with_uri (choice, uri, parent_window, writable);
-      update_permissions_store (request->app_id, content_type, choice);
+      if (launch_application_with_uri (choice, uri, parent_window, writable))
+        update_permissions_store (xdp_app_info_get_id (request->app_info), content_type, choice);
     }
 
 out:
@@ -452,7 +461,7 @@ handle_open_in_thread_func (GTask *task,
 {
   Request *request = (Request *)task_data;
   const char *parent_window;
-  const char *app_id = request->app_id;
+  const char *app_id = xdp_app_info_get_id (request->app_info);
   g_autofree char *uri = NULL;
   g_autoptr(GError) error = NULL;
   g_autoptr(XdpImplRequest) impl_request = NULL;
@@ -485,7 +494,9 @@ handle_open_in_thread_func (GTask *task,
           if (request->exported)
             {
               g_variant_builder_init (&opts_builder, G_VARIANT_TYPE_VARDICT);
-              xdp_request_emit_response (XDP_REQUEST (request), 2, g_variant_builder_end (&opts_builder));
+              xdp_request_emit_response (XDP_REQUEST (request),
+                                         XDG_DESKTOP_PORTAL_RESPONSE_OTHER,
+                                         g_variant_builder_end (&opts_builder));
               request_unexport (request);
             }
           return;
@@ -493,82 +504,23 @@ handle_open_in_thread_func (GTask *task,
     }
   else
     {
-      g_autofree char *proc_path = NULL;
-      int fd_flags;
-      char path_buffer[PATH_MAX + 1];
-      g_autofree char *rewritten_path = NULL;
-      char *path;
-      ssize_t symlink_size;
-      struct stat st_buf;
-      struct stat real_st_buf;
+      g_autofree char *path = NULL;
 
-      proc_path = g_strdup_printf ("/proc/self/fd/%d", fd);
-
-      if (fd == -1 ||
-          (fd_flags = fcntl (fd, F_GETFL)) == 0 ||
-          ((fd_flags & O_PATH) != O_PATH) ||
-          ((fd_flags & O_NOFOLLOW) == O_NOFOLLOW) ||
-          fstat (fd, &st_buf) < 0 ||
-          (st_buf.st_mode & S_IFMT) != S_IFREG ||
-          (symlink_size = readlink (proc_path, path_buffer, PATH_MAX)) < 0)
+      path = xdp_app_info_get_path_for_fd (request->app_info, fd);
+      if (path == NULL)
         {
           /* Reject the request */
           if (request->exported)
             {
               g_variant_builder_init (&opts_builder, G_VARIANT_TYPE_VARDICT);
-              xdp_request_emit_response (XDP_REQUEST (request), 2, g_variant_builder_end (&opts_builder));
+              xdp_request_emit_response (XDP_REQUEST (request),
+                                         XDG_DESKTOP_PORTAL_RESPONSE_OTHER,
+                                         g_variant_builder_end (&opts_builder));
               request_unexport (request);
             }
           return;
         }
 
-      path_buffer[symlink_size] = 0;
-
-      path = path_buffer;
-
-      if (request->app_info != NULL)
-        {
-          if (g_str_has_prefix (path, "/newroot/usr/"))
-            {
-              g_autofree char * usr_root =
-                g_key_file_get_string (request->app_info,
-                                       "Instance", "runtime-path", NULL);
-              if (usr_root)
-                {
-                  rewritten_path = g_build_filename (usr_root, path + strlen ("/newroot/usr/"), NULL);
-                  path = rewritten_path;
-                }
-            }
-          else if (g_str_has_prefix (path, "/newroot/app/"))
-            {
-              g_autofree char * app_root =
-                g_key_file_get_string (request->app_info,
-                                       "Instance", "app-path", NULL);
-              if (app_root)
-                {
-                  rewritten_path = g_build_filename (app_root, path + strlen ("/newroot/app/"), NULL);
-                  path = rewritten_path;
-                }
-            }
-          else if (g_str_has_prefix (path, "/newroot/"))
-            path = path + strlen ("/newroot");
-
-          /* Verify that this is the same file as the app opened */
-          if (stat (path, &real_st_buf) < 0 ||
-              st_buf.st_dev != real_st_buf.st_dev ||
-              st_buf.st_ino != real_st_buf.st_ino)
-            {
-              /* Different files on the inside and the outside, reject the request */
-              if (request->exported)
-                {
-                  g_variant_builder_init (&opts_builder, G_VARIANT_TYPE_VARDICT);
-                  xdp_request_emit_response (XDP_REQUEST (request), 2, g_variant_builder_end (&opts_builder));
-                  request_unexport (request);
-                }
-              return;
-            }
-        }
-
       get_content_type_for_file (path, &content_type);
       basename = g_path_get_basename (path);
 
@@ -583,15 +535,16 @@ handle_open_in_thread_func (GTask *task,
   if (skip_app_chooser || (!always_ask && (latest_count >= latest_threshold)))
     {
       /* If a recommended choice is found, just use it and skip the chooser dialog */
-      launch_application_with_uri (skip_app_chooser ? choices[0] : latest_id,
-                                   uri,
-                                   parent_window,
-                                   writable);
-
+      gboolean result = launch_application_with_uri (skip_app_chooser ? choices[0] : latest_id,
+                                                     uri,
+                                                     parent_window,
+                                                     writable);
       if (request->exported)
         {
           g_variant_builder_init (&opts_builder, G_VARIANT_TYPE_VARDICT);
-          xdp_request_emit_response (XDP_REQUEST (request), 0, g_variant_builder_end (&opts_builder));
+          xdp_request_emit_response (XDP_REQUEST (request),
+                                     result ? XDG_DESKTOP_PORTAL_RESPONSE_SUCCESS : XDG_DESKTOP_PORTAL_RESPONSE_OTHER,
+                                     g_variant_builder_end (&opts_builder));
           request_unexport (request);
         }
       return;
@@ -613,6 +566,8 @@ handle_open_in_thread_func (GTask *task,
   g_variant_builder_add (&opts_builder, "{sv}", "content_type", g_variant_new_string (content_type));
   if (basename)
     g_variant_builder_add (&opts_builder, "{sv}", "filename", g_variant_new_string (basename));
+  if (uri)
+    g_variant_builder_add (&opts_builder, "{sv}", "uri", g_variant_new_string (uri));
 
   impl_request = xdp_impl_request_proxy_new_sync (g_dbus_proxy_get_connection (G_DBUS_PROXY (impl)),
                                                   G_DBUS_PROXY_FLAGS_NONE,
